'use strict';

Object.defineProperty(exports, "__esModule", {
				value: true
});

var _eos = require('./eos');

var eos = _interopRequireWildcard(_eos);

var _murmurhash = require('murmurhash');

var _murmurhash2 = _interopRequireDefault(_murmurhash);

var _Reputable = require('../models/Reputable');

var _Reputable2 = _interopRequireDefault(_Reputable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const fingerprinted = str => _murmurhash2.default.v2(str);

const getReputation = (() => {
				var _ref = _asyncToGenerator(function* (reputable) {
								const reputation = yield eos.read({
												table: 'reputations',
												scope: reputable.fingerprint,
												limit: 1,
												firstOnly: true
								}).catch(function () {
												return null;
								});

								const fragTotals = yield Promise.all(reputation.fragments.map((() => {
												var _ref2 = _asyncToGenerator(function* (x) {
																return eos.read({
																				table: 'fragtotal',
																				scope: fingerprinted(x.type),
																				limit: 1,
																				firstOnly: true
																}).catch(function () {
																				return null;
																});
												});

												return function (_x2) {
																return _ref2.apply(this, arguments);
												};
								})()));

								const parseAsset = function (asset) {
												return parseFloat(asset.split(' ')[0]);
								};

								reputation.fragments.map(function (frag) {
												const fragTotal = fragTotals.find(function (x) {
																return x.type === frag.type;
												});
												frag.reputation = 0;

												const up = parseAsset(frag.up);
												const down = parseAsset(frag.down);
												const tup = parseAsset(fragTotal.up);
												const tdown = parseAsset(fragTotal.down);

												frag.reputation = parseFloat(((up > 0 ? up / tup : 0) - (down > 0 ? down / tdown : 0)).toFixed(4));
												frag.fingerprint = fragTotal.fingerprint;
								});

								delete reputation.fingerprint;
								return _Reputable.Reputation.fromJson(reputation);
				});

				return function getReputation(_x) {
								return _ref.apply(this, arguments);
				};
})();

class ReputationService {

				constructor() {}

				repute(username, entity, fragments, details = '') {
								return _asyncToGenerator(function* () {
												if (!username.length) throw new Error('Invalid username');
												if (!entity.length || entity.indexOf('::') === -1) throw new Error('Invalid entity');
												if (!fragments.every(function (frag) {
																return frag instanceof _Reputable.Fragment && frag.validate();
												})) throw new Error('Invalid fragments');
												return eos.contract.repute(username, entity, fragments, details, eos.options);
								})();
				}

				votetype(username, type) {
								return _asyncToGenerator(function* () {
												return eos.contract.votetype(username, type, eos.options);
								})();
				}

				getEntity(entity) {
								return _asyncToGenerator(function* () {

												const reputable = yield eos.read({
																table: 'reputables',
																index: fingerprinted(entity),
																limit: 1,
																firstOnly: true,
																model: _Reputable2.default
												}).catch(function () {
																return null;
												});

												if (reputable) reputable.reputation = yield getReputation(reputable);

												return reputable;
								})();
				}

				searchForEntity(name) {
								return _asyncToGenerator(function* () {

												const reputables = yield eos.read({
																table: 'reputables',
																index: fingerprinted(name),
																key_type: 'i64',
																index_position: 2,
																limit: 500,
																rowsOnly: true,
																model: _Reputable2.default
												}).catch(function () {
																return null;
												});

												if (reputables.length) {
																yield Promise.all(reputables.map((() => {
																				var _ref3 = _asyncToGenerator(function* (reputable) {
																								reputable.reputation = yield getReputation(reputable);
																								return true;
																				});

																				return function (_x3) {
																								return _ref3.apply(this, arguments);
																				};
																})()));
												}

												return reputables;
								})();
				}

				getFragments(base = 0) {
								return _asyncToGenerator(function* () {
												return eos.read({
																table: 'reptypes',
																key_type: 'i64',
																index_position: 2,
																index: base,
																search: 1,
																limit: 100,
																rowsOnly: true,
																model: _Reputable.RepType
												}).catch(function () {
																return [];
												});
								})();
				}

				getFragmentsFor(reputable = null) {
								var _this = this;

								return _asyncToGenerator(function* () {
												const globalFragments = yield _this.getFragments();
												const basedFragments = reputable ? (yield _this.getFragments(reputable.fingerprint)).map(function (x) {
																x.isBased = true;
																return x;
												}) : [];
												return globalFragments.concat(basedFragments);
								})();
				}
}
exports.default = ReputationService;